@@ -1,16 +1,155 @@
 from sqlalchemy_traversal.interfaces    import ISABase
 from sqlalchemy_traversal.interfaces    import ISASession
 from sqlalchemy_traversal.interfaces    import ISaver
+from sqlalchemy_traversal.interfaces    import IAfterSaver
 
 from datetime                           import datetime
 from datetime                           import date
 from datetime                           import time
 
 from sqlalchemy.orm                     import class_mapper
+from sqlalchemy.exc                     import InvalidRequestError
+from sqlalchemy                         import not_
+from zope.interface                     import providedBy
 
 import colander
 import venusian
 
+def filter_list_by_qs(qs, collection):
+    """ This function takes a querystring and a list
+
+    Accepted QS arguments are: __order_by and property names, for example:
+
+        /conference/1/fields?__order_by=sort_order&name=Foo Con
+
+    You can also do an in query:
+
+        /conference?pk.in=1,2
+
+    You can also do a not query by using pk.not=
+
+        /conference?name.not=PyCon2012
+
+    You can also do a not in query by using pk.not=
+
+        /conference?name.notin=PyCon2012,PyCon2011
+    """
+    method = 'asc'
+
+    order_by = None
+
+    if '__order_by' in qs:
+        order_by = qs.pop('__order_by')
+
+    if order_by:
+        orders = [x.strip() for x in order_by.split(',')]
+
+        for order in orders:
+            if ' ' in order:
+                order, method = order.split()
+                if method == 'asc':
+                    collection.sort(key=lambda x: getattr(x, order))
+                else:
+                    collection.sort(key=lambda x: getattr(x, order), reverse=True)
+
+
+    for key, value in qs.iteritems():
+        if '.in' in key:
+            key = key[0:-3]
+            values = value.split(',')
+
+            for obj in collection[:]:
+                if not getattr(obj, key) in values:
+                    collection.remove(obj)
+
+        elif '.notin' in key:
+            key = key[0:-3]
+            values = value.split(',')
+
+            for obj in collection[:]:
+                if getattr(obj, key) in values:
+                    collection.remove(obj)
+
+        elif '.not' in key:
+            key = key[0:-4]
+
+            for obj in collection[:]:
+                if getattr(obj, key) == value:
+                    collection.remove(obj)
+        else:
+            for obj in collection[:]:
+                if not getattr(obj, key) == value:
+                    collection.remove(obj)
+
+    return collection
+
+def filter_query_by_qs(session, cls, qs, existing_query=None):
+    """ This function takes a SA Session, a SA ORM class, and a 
+    query string that it can filter with.
+
+    Accepted QS arguments are: __order_by and property names, for example:
+
+        /conference?__order_by=sort_order&name=Foo Con
+
+    You can also do an in query:
+
+        /conference?pk.in=1,2
+
+    You can also do a not query by using pk.not=
+
+        /conference?name.not=PyCon2012
+
+    You can also do a not in query by using pk.not=
+
+        /conference?name.notin=PyCon2012,PyCon2011
+    """
+    if existing_query:
+        query = existing_query
+    else:
+        query = session.query(cls)
+
+    method = 'asc'
+
+    order_by = None
+
+    if '__order_by' in qs:
+        order_by = qs.pop('__order_by')
+
+    if order_by:
+        orders = [x.strip() for x in order_by.split(',')]
+
+        for order in orders:
+            if ' ' in order:
+                order, method = order.split()
+
+            prop = getattr(cls, order)
+
+            if method == 'desc':
+                query = query.order_by(prop.desc())
+            else:
+                query = query.order_by(prop.asc())
+
+    for key, value in qs.iteritems():
+        if '.in' in key:
+            key = key[0:-3]
+            prop = getattr(cls, key)
+            query = query.filter(prop.in_(value.split(',')))
+        elif '.notin' in key:
+            key = key[0:-6]
+            prop = getattr(cls, key)
+            query = query.filter(not_(prop.in_(value.split(','))))
+        elif '.not' in key:
+            key = key[0:-4]
+            prop = getattr(cls, key)
+            query = query.filter(prop != value)
+        else:
+            prop = getattr(cls, key)
+            query = query.filter(prop == value)
+
+    return query
+
+
+
 def format_colander_errors(e):
     """
     This formats our colander errors in a nice format
@@ -150,6 +289,7 @@ class JsonSerializableMixin(TraversalBase):
 
             # format and date/datetime/time properties to isoformat
             obj = getattr(self, key)
+
             if isinstance(obj, (datetime, date, time)):
                 props[key] = obj.isoformat()
                 continue
@@ -204,7 +344,7 @@ class ModelCollection(TraversalBase):
                 # Does the item in the collection have the lookup_key we
                 # are looking for?
                 if data == compare_value:
-                    obj.__parent__ = self.__parent__
+                    obj.__parent__ = self
                     if hasattr(self, '_request'):
                         obj._request = self._request
 
@@ -220,6 +360,28 @@ class ModelCollection(TraversalBase):
     def __json__(self, request):
         return [self.try_to_json(request, x) for x in self.collection]
 
+def recurse_get_traversal_root(obj):
+    if hasattr(obj, 'get_class'):
+        return obj
+    else:
+        return recurse_get_traversal_root(obj.__parent__)
+
+def get_prop_from_cls(cls, attr):
+    """
+    Gets an sa mapper class to look up properties on
+    we need to find out which class the attribute is
+    attached to and return that
+    """
+    try:
+        mapper = class_mapper(cls, compile=False)
+        rel_prop = mapper.get_property(attr)
+        name = rel_prop.target.name
+        root = recurse_get_traversal_root(cls)
+        new_cls = root.get_class(name)
+
+        return new_cls
+    except InvalidRequestError:
+        raise KeyError
 
 class TraversalMixin(JsonSerializableMixin):
     """
@@ -227,19 +389,13 @@ class TraversalMixin(JsonSerializableMixin):
     """
     _traversal_lookup_key = 'id'
 
-    def _recurse_get_traversal_root(self, obj):
-        if hasattr(obj, 'get_class'):
-            return obj
-        else:
-            return self._recurse_get_traversal_root(obj.__parent__)
-
     def _get_class(self, name):
         """
         This is a recursive function that will search every parent
         until we get to the TraversalRoot so that we can get the
         class name
         """
-        root = self._recurse_get_traversal_root(self)
+        root = recurse_get_traversal_root(self)
 
         return root.get_class(name)
 
@@ -259,33 +415,37 @@ class TraversalMixin(JsonSerializableMixin):
             # POST means "create", so we are always looking for a class
             if self._request.method == 'POST' and \
                     self._request.path.endswith(attribute):
-                # get an sa mapper class to look up properties on
-                # we need to find out which class the attribute is
-                # attached to and return that
-                mapper = class_mapper(self.__class__, compile=False)
-                rel_prop = mapper.get_property(attribute)
-                name = rel_prop.target.name
-                cls = self._get_class(name)()
+
+                cls = get_prop_from_cls(self.__class__, attribute)
+
                 cls.__parent__ = self
                 cls._request = self._request
 
-                return cls
+                return cls()
 
         obj = getattr(self, attribute)
 
         # The model had the specific attribute, so we just need to figure out
         # if we are returning a collection or a single instance
         if obj != None:
+            request = None
+            if hasattr(self, '_request'):
+                request = self._request
+
+#            if request:
+                #session = get_session(request)
+                #filter_query_by_qs(session, obj, self._request.GET)
+
             try:
-                ignore_types = (str, unicode, int, float,TraversalMixin)
+                ignore_types = (str, unicode, int, float, TraversalMixin)
                 if not isinstance(obj, ignore_types):
                     # is this is a collection
                     iter(obj)
-                    col = ModelCollection(obj)
+                    col = ModelCollection(filter_list_by_qs(self._request.GET, obj))
                     col.__parent__ = self
 
-                    if hasattr(self, '_request'):
-                        col._request = self._request
+                    if request:
+                        col._request = request
 
                     return col
             except TypeError as e:
@@ -303,9 +463,10 @@ class register_save(object):
     If an API is called with a POST or PUT it will first try to run validation
     against the schema before doing anything
     """
-    def __init__(self, cls, schema):
+    def __init__(self, cls, schema, exception_handlers=None):
         self.cls = cls
         self.schema = schema
+        self.exception_handlers = exception_handlers
 
     def register(self, scanner, name, wrapped):
         def save(request):
@@ -335,8 +496,26 @@ class register_save(object):
             for key, value in result.iteritems():
                 setattr(request.context, key, value)
 
-            session.add(request.context)
-            session.flush()
+            try:
+                session.add(request.context)
+                session.flush()
+            except Exception as e:
+                error_dict = {'has_errors': True}
+                if self.exception_handlers:
+                    if e.__class__ in self.exception_handlers:
+                        new_errors = self.exception_handlers[e.__class__](request.context, e)
+
+                        return dict(error_dict.items() + new_errors.items())
+
+                error_dict['message']  = e.message
+                return error_dict
+
+            after_save = request.registry.adapters.lookup(
+                [providedBy(request.context)], IAfterSaver
+            )
+
+            if after_save:
+                after_save(request)
 
             return request.context
 
@@ -348,6 +527,26 @@ class register_save(object):
 
         return wrapped
 
+class register_after_save(object):
+    """
+    This is a decorator that matches a Model class and will execute after
+    the save has flushed
+    """
+    def __init__(self, cls):
+        self.cls = cls
+
+    def register(self, scanner, name, wrapped):
+        def after_save(request):
+            wrapped(request)
+
+        registry = scanner.config.registry
+        registry.registerAdapter(after_save, (self.cls, ), IAfterSaver)
+
+    def __call__(self, wrapped):
+        venusian.attach(wrapped, self.register)
+
+        return wrapped
+
 def includeme(config):
     config.scan('sqlalchemy_traversal')
     config.include('sqlalchemy_traversal.routes')
